/*-----------------------------------------------------------------------------
Modul Name : CofigFile.h
Date       : 09.03.2001
-------------------------------------------------------------------------------
Description: Class for Loading and Manipulating with Server Config File

Comments   : _
-----------------------------------------------------------------------------*/
#ifndef __CONFIGFILE_H__
#define __CONFIGFILE_H__

#include "RealPoint.h"

// Configurations for soccerserver
class CConfigFile 
{
public:
/********************* methods ***********************************************/
        CConfigFile();

/********************* atributes *********************************************/
        // width of goals (default 7.32)
        float goal_width;
        // size, decay, random parameter, weight, maximum speed of players
        // (default 1.0, 0.5, 0.1, 60.0, 32.0)
        float player_size;
        float player_decay;
        float player_rand;
        float player_weight;
        float player_speed_max;

        // maximum and recovery step of player's stamina (default 2500.0, 50.0)
        float stamina_max;
        // 35.0
        float stamina_inc_max;

        // decriment threshold ,decriment step and minimum of player's recovery
        // (default 0.3, 0.05, 0.1)
        float recover_dec_thr;
        float recover_dec;
        float recover_min;

        // decriment threshold, decriment step, incriment threshold, incriment step
        // and minimum of player's effort (default 0.4, 0.05, 0.9, 0.05, 0.1)
        float effort_dec_thr;
        float effort_dec;
        float effort_inc_thr;
        float effort_inc;
        float effort_min;

        // maximum, recovery step and decay of player's hear capacity (default 2, 1, 1)
        float hear_max;
        float hear_inc;
        float hear_decay;

        // inertia moment of player (default 5.0)
        float inertia_moment;

        // interval of sense_body (default 100)
        float sense_body_step;

        // goalie catchable area length, width (default 2.0, 1.0)
        float catchable_area_l;
        float catchable_area_w;

        // goalie catchbale probability (default 1.0)
        float catch_probability;

        // goalie catch ban cycle (default 5)
        int catch_ban_cycle;

        // size, decay, random parameter, weight and maximum speed of a ball
        // (default 0.15, 0.8, 0.2, 0.2, 32.0)
        float ball_size;
        float ball_decay;
        float ball_rand;
        float ball_weight;
        float ball_speed_max;

        // force, direction and random parameter of wind (default 10.0, 0.0, 0.3)
        float wind_force;
        float wind_dir;
        float wind_rand;

        // kickable margin (default 1.0) kickable_area = kickable_margin + bsize + psize
        float kickable_margin;

        // corner kick margin (default 1.0)
        float ckick_margin;

        // magnification of power in dash, kick  (default 0.1, 0.1)
        float dash_power_rate;
        //kick_power_rate   : 0.013
        float kick_power_rate;

        // angle of view corn [unit: degree] (default 90.0)
        float visible_angle;

        // audio cut off distance (default 50.0)
        float audio_cut_dist;

        // quantize step of distance for move_object, landmark (default 0.1, 0.01)
        float quantize_step;
        float quantize_step_l;

        // max and min of power in dash and kick (default 100, -30)
        float maxpower;
        float minpower;

        // max and min of power in turn (default 180, -180)
        float maxmoment;
        float minmoment;

        // max and min of power in turn_neck (default 180,-180)
        float maxneckmoment;
        float minneckmoment;

        // max and min of neck angle (default 90, -90)
        float maxneckang;
        float minneckang;

        // Default port number (default 6000)
        int port;

        // Default coach port number (default 6001)
        int coach_port;

        // Default upper limit of the number of online coach's message (default 128)
        int say_coach_cnt_max;

        // Default upper limit of length of online coach's message (default 128)
        int say_coach_msg_size;

        // Default interval of online coach's look (default 100)
        unsigned send_vi_step;

        // time step of simulation [unit:msec] (default 100)
        unsigned simulator_step;

        // time step of visual information [unit:msec] (default 150)
        unsigned send_step;

        // time step of acception of command [unit: msec] (defalut 20)
        unsigned recv_step;

        // length of half of game [unit:sec] (default 6000)
        // (if negative, a game does not stop automatically)
        int half_time;
        //half_time     : 30000

        // string size of say message [unit:byte] (default 256)
        int say_msg_size;

        // flag for using off side rule. [on/off] (default on)
        //use_offside       : on
        int use_offside;

        // offside active area size (default 9.15)
        //offside_active_area_size  : 9.15
        float offside_active_area_size;

        // forbid kick off offside. [on/off] (default on)
        int forbid_kick_off_offside;

        // flag for verbose mode. [on/off] (default off)
        int verbose;

        // flag for record log. (default 2)
        // (old version 1)
        float record_version;

        // flag for record client command log. [on/off] (default off)
        int record_log;

        // flag for send client command log. [on/off] (default on)
        int send_log;

        // offside kick margin. (default 9.15)
        		
		float offside_kick_margin;

//****************************************************

		CString defaultTeamName();
		CString defaultServerName();
		long defaultServerSocket();
		int defaultGoalie();

		CRealPoint defaultPlayerPos(int UniformNum, BOOL side, BOOL goalie);
		void loadPositions(void);

		//vrati TRUE ak sa podari nacitat z .ini nazov timu, inak FALSE
		BOOL isConfigReady();
private:
	CString TeamName, ServerName;
	long ServerSocket;
	int Goalie;
	CRealPoint pp[12];		// hracske pozicie

	CString getProfileString(LPCTSTR lpSection, LPCTSTR lpKey, LPCTSTR lpDefault);
	long getProfileInt(LPCTSTR lpSection, LPCTSTR lpKey, unsigned int aiDefault);

};

#endif