// LowAgent.cpp: implementation of the CLowAgent class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Sharks.h"

#include "SharkTypes.h"
#include "LowAgent.h"

#include "NetDlg.h"
#include "TParseString.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CLowAgent::CLowAgent()
{
	m_IsConnected=false;
	m_TryingToConnect=false;
	m_GoalPerson=FALSE;

	m_DebugMode=FALSE;
	m_DebugTicks=0;
	m_DebugRecord=FALSE;
	m_DebugLogIndex=0;
}

CLowAgent::~CLowAgent()
{

	if(m_IsConnected)
	{
		CString Bm;
		Bm="(bye)";
		SendMsg(Bm);
	}
}

BOOL CLowAgent::ConnectAgentEx(CString	TeamName,
							   CString SocServer,
							   UINT	SocPort,
							   BOOL	GoalPerson)
{
	BOOL napojeny;

	napojeny=Create(SocPort, SocServer);
	if (! napojeny) 
	{
		OsetriSokError(GetLastError());
		return FALSE;
	}
	
	CString Im;
	if(GoalPerson)
	{
		Im.Format("(init %s (version 5.00) (goalie))",(LPCTSTR) TeamName);
	}
	else
	{
		Im.Format("(init %s (version 5.00))",(LPCTSTR) TeamName);
	}

	SendMsg(Im);

	m_TryingToConnect=TRUE;
	m_NazovTimu=TeamName;
	m_GoalPerson=GoalPerson;

	return true;
}


/*
	Posle serveru spravu
*/
void CLowAgent::SendMsg(CString& msg)
{
	if(m_DebugMode) return;
	int retval;
	retval=SendTo(msg,msg.GetLength(),GetPort(),GetAddress(),0);

	if(retval==SOCKET_ERROR) OsetriSokError(GetLastError());
}

/*
	Vypise chybu soketu
*/
void CLowAgent::OsetriSokError(int e)
{
	LPSTR NazCH;
	LPSTR PoPCH;
	char pombuf[128];

	switch(e)
	{
	case WSANOTINITIALISED:
		PoPCH="WSANOTINITIALISED";
		NazCH="A successful AfxSocketInit must occur before using this API.";
		break;
	case WSAENETDOWN:
		PoPCH="WSAENETDOWN";
		NazCH="The Windows Sockets implementation detected that the network subsystem failed.";
		break;
	case WSAEACCES:
		PoPCH="WSAEACCES";
		NazCH="The requested address is a broadcast address, but the appropriate flag was not set.";
		break;
	case WSAEINPROGRESS:
		PoPCH="WSAEINPROGRESS";
		NazCH="A blocking Windows Sockets operation is in progress.";
		break;
	case WSAEFAULT:
		PoPCH="WSAEFAULT";
		NazCH="The lpBuf or lpSockAddr parameters are not part of the user address space, or the lpSockAddr argument is too small (less than the size of a SOCKADDR structure).";
		break;
	case WSAEINVAL:
		PoPCH="WSAEINVAL";
		NazCH="The host name is invalid.";
		break;
	case WSAENETRESET:
		PoPCH="WSAENETRESET";
		NazCH="The connection must be reset because the Windows Sockets implementation dropped it.";
		break;
	case WSAENOBUFS:
		PoPCH="WSAENOBUFS";
		NazCH="The Windows Sockets implementation reports a buffer deadlock.";
		break;
	case WSAENOTCONN:
		PoPCH="WSAENOTCONN";
		NazCH="The socket is not connected.";
		break;
	case WSAENOTSOCK:
		PoPCH="WSAENOTSOCK";
		NazCH="The descriptor is not a socket.";
		break;
	case WSAEOPNOTSUPP:
		PoPCH="WSAEOPNOTSUPP";
		NazCH="MSG_OOB was specified, but the socket is not of type SOCK_STREAM.";
		break;
	case WSAESHUTDOWN:
		PoPCH="WSAESHUTDOWN";
		NazCH="The socket has been shut down; it is not possible to call SendTo on a socket after ShutDown has been invoked with nHow set to 1 or 2.";
		break;
	case WSAEWOULDBLOCK:
		PoPCH="WSAEWOULDBLOCK";
		NazCH="The socket is marked as nonblocking and the requested operation would block.";
		break;
	case WSAEMSGSIZE:
		PoPCH="WSAEMSGSIZE";
		NazCH="The socket is of type SOCK_DGRAM, and the datagram is larger than the maximum supported by the Windows Sockets implementation.";
		break;
	case WSAECONNABORTED:
		PoPCH="WSAECONNABORTED";
		NazCH="The virtual circuit was aborted due to timeout or other failure.";
		break;
	case WSAECONNRESET:
		PoPCH="WSAECONNRESET";
		NazCH="The virtual circuit was reset by the remote side.";
		break;
	case WSAEADDRNOTAVAIL:
		PoPCH="WSAEADDRNOTAVAIL";
		NazCH="The specified address is not available from the local machine.";
		break;
	case WSAEAFNOSUPPORT:
		PoPCH="WSAEAFNOSUPPORT";
		NazCH="Addresses in the specified family cannot be used with this socket.";
		break;
	case WSAEDESTADDRREQ:
		PoPCH="WSAEDESTADDRREQ";
		NazCH="A destination address is required.";
		break;
	case WSAENETUNREACH:
		PoPCH="WSAENETUNREACH";
		NazCH="The network cannot be reached from this host at this time.";
		break;
	default:
		PoPCH="Unknown error";
		sprintf(pombuf,"%x",e);
		NazCH=pombuf;
		break;
	}

	MessageBox(NULL,NazCH,PoPCH,MB_ICONSTOP);
}

void CLowAgent::OnReceive(int nErrorCode)
{
	if(nErrorCode)
	{
		OsetriSokError(nErrorCode);
		return;
	}

	static char buf[4096];
	int retval;
	UINT port=GetPort();
	CString Address=GetAddress();

	retval=ReceiveFrom(buf,4095,Address,port,0);
	SetPort(port);

	if(retval!=SOCKET_ERROR)
	{
		buf[retval]=0;
		ParseReceive(buf);

		if(m_DebugRecord && !m_DebugMode)
		{//ukladanie logu
			m_DebugLog.addEvent(GetTickCount(),buf);
		}
	}
	else
	{
		OsetriSokError(GetLastError());
	}
}

void CLowAgent::ParseHear(LPCSTR message)
{
	TParseString msg(message);
	CString tmp;
	int heartime;
	int mode=PM_None;
	char* keciRozhodcu[]=PLAYMODE_STRINGS;

	int direction;

        // get the time
        msg.SetPosition(5);
        heartime = msg.ParseInteger();
        if (msg.HasErrorOccured())
		{
			MsgB("!");
			return;
		}

        // get the source
        switch (msg.GetAt(msg.GetPosition()))
        {
        case 'r':
                // it is from the referee
                if (!msg.ParseString(tmp))
                {
					MsgB("!");
					return;
				}
                // get the message
                if (!msg.ParseString(tmp))
                {
					MsgB("!");
					return;
				}

                while ((mode < NO_OF_PLAYMODES) && (tmp.Compare(keciRozhodcu[mode])))
                        mode++;

                if (mode < NO_OF_PLAYMODES)
				{
                    Handler_ChangeMode((PlayMode)mode,heartime);
				}
				else
				{
					switch(tmp.GetAt(0))
					{
					case 'g': //goal_Side_UniformNumber
						switch(tmp.GetAt(5))
						{
						case 'l':
							Handler_ChangeMode(PM_GOAL_L,heartime);
							break;
						case 'r':
							Handler_ChangeMode(PM_GOAL_R,heartime);
							break;
						}
						break;
					//case 'f': //foul_Side_Point
						//break;
					}

					CString rmsg;
					rmsg.Format("%s\n",message);
					cPrint(rmsg,9);
				}
                break;

        case 's':
                // it has been sent by myself
                break;

        default: //pocujem
                direction=msg.ParseInteger();
				if (msg.HasErrorOccured())
				{
					MsgB("!");
					return;
				}

				if(!msg.ParseString(tmp))
				{
					MsgB("!");
					return;
				}
				Handler_Hear(direction,tmp,heartime);
                break;
        }


}

void CLowAgent::ParseSee(LPCSTR message)
{
	TParseString    msg(message);
	TParseString    strObject;
	int seetime;

	// get the time
	msg.SetPosition(5);
	seetime = msg.ParseInteger();
	if (msg.HasErrorOccured())
	{
		MsgB("!!!");
        return;
	}

	static Ciara poleCiary[4];
	unsigned poc_ciara=0;
	ZeroMemory(poleCiary,sizeof(poleCiary));
	
	static Flag poleFlags[MAX_FLAG];
	unsigned poc_flag=0;
	ZeroMemory(poleFlags,sizeof(poleFlags));

	static Player nasiHraci[10];
	unsigned poc_nasich=0;
	ZeroMemory(nasiHraci,sizeof(nasiHraci));

	static Player cudziHraci[11];
	unsigned poc_cudzich=0;
	ZeroMemory(cudziHraci,sizeof(cudziHraci));

	static Player neutralHraci[21];
	unsigned poc_neutral=0;
	ZeroMemory(neutralHraci,sizeof(neutralHraci));

	static Player Lopta;
	ZeroMemory(&Lopta,sizeof(Lopta));

	double dist;
	double dir;

	double distchng;
	BOOL b_distchng=false;

	double dirchng;
	BOOL b_dirchng=false;

	double facedir;
	BOOL b_facedir=false;

	double bodydir;
	BOOL b_bodydir=false;

	CString teamname;

	TParseString name;
	CString T;
	BOOL nashrac;
	DWORD properties;


	while(msg.ParseBracket(strObject))
	{
		strObject.SetPosition(0);
		name.SetPosition(0);
		strObject.ParseBracket(name);
		
		b_distchng=false;
		b_dirchng=false;
		b_facedir=false;
		b_bodydir=false;

		facedir=0;
		bodydir=0;

		dist=strObject.ParseDouble();
		dir=strObject.ParseDouble();

		distchng=strObject.ParseDouble();
		if(!strObject.HasErrorOccured())
		{
			b_distchng=true;
			dirchng=strObject.ParseDouble();
			b_dirchng=true;

			facedir=strObject.ParseDouble();
			if(!strObject.HasErrorOccured())
			{
				b_facedir=true;
				bodydir=strObject.ParseDouble();
				b_bodydir=true;
			}
		}
		//analyza objektu name

		switch(name.GetAt(0))
		{
		case 'P': //neznamy hrac za mnou
			neutralHraci[poc_neutral].properties=P_DISTANCE|P_DIRECTION;
			neutralHraci[poc_neutral].distance=dist;
			neutralHraci[poc_neutral].direction=dir;
			if(b_distchng)
			{
				neutralHraci[poc_neutral].properties|=P_DISTCHNG;
				neutralHraci[poc_neutral].distchng=distchng;
			}
			if(b_dirchng)
			{
				neutralHraci[poc_neutral].properties|=P_DIRCHNG;
				neutralHraci[poc_neutral].dirchng=dirchng;
			}
			if(b_facedir)
			{
				neutralHraci[poc_neutral].properties|=P_FACEDIR;
				neutralHraci[poc_neutral].faceDir=facedir;
			}
			if(b_bodydir)
			{
				neutralHraci[poc_neutral].properties|=P_BODYDIR;
				neutralHraci[poc_neutral].bodyDir=bodydir;
			}
			poc_neutral++;
			break;
		case 'p': //hrac v zornom poli
			properties=P_DISTANCE|P_DIRECTION;
			if(b_distchng) properties|=P_DISTCHNG;
			if(b_dirchng) properties|=P_DIRCHNG;
			if(b_facedir) properties|=P_FACEDIR;
			if(b_bodydir) properties|=P_BODYDIR;

			if(name.GetLength()>7)
			{
			name.SetPosition(7);
			name.ParseString(teamname);

				if(m_NazovTimu.Compare(teamname)==0)
					nashrac=true;
				else
					nashrac=false;

				int Ch=name.ParseInteger();
				if(!name.HasErrorOccured())
				{
					properties|=P_NUMBER;
				}

				if(nashrac)
				{
					nasiHraci[poc_nasich].distance=dist;
					nasiHraci[poc_nasich].direction=dir;
					nasiHraci[poc_nasich].distchng=distchng;
					nasiHraci[poc_nasich].dirchng=dirchng;
					nasiHraci[poc_nasich].faceDir=facedir;
					nasiHraci[poc_nasich].bodyDir=bodydir;
					nasiHraci[poc_nasich].UniformNumber=Ch;

					nasiHraci[poc_nasich].properties=properties;
					poc_nasich++;
				}
				else
				{
					cudziHraci[poc_cudzich].distance=dist;
					cudziHraci[poc_cudzich].direction=dir;
					cudziHraci[poc_cudzich].distchng=distchng;
					cudziHraci[poc_cudzich].dirchng=dirchng;
					cudziHraci[poc_cudzich].faceDir=facedir;
					cudziHraci[poc_cudzich].bodyDir=bodydir;
					cudziHraci[poc_cudzich].UniformNumber=Ch;

					cudziHraci[poc_cudzich].properties=properties;
					poc_cudzich++;
				}
			}
			else //neznamy hrac
			{
				neutralHraci[poc_neutral].distance=dist;
				neutralHraci[poc_neutral].direction=dir;
				neutralHraci[poc_neutral].distchng=distchng;
				neutralHraci[poc_neutral].dirchng=dirchng;
				neutralHraci[poc_neutral].faceDir=facedir;
				neutralHraci[poc_neutral].bodyDir=bodydir;

				neutralHraci[poc_neutral].properties=properties;
				poc_neutral++;
			}
			break;
		case 'B':
		case 'b': //lopta
			Lopta.distance=dist;
			Lopta.direction=dir;
			Lopta.properties=P_DISTANCE|P_DIRECTION;
			if(b_distchng)
			{
				Lopta.properties|=P_DISTCHNG;
				Lopta.distchng=distchng;
			}
			if(b_dirchng)
			{
				Lopta.properties|=P_DIRCHNG;
				Lopta.dirchng=dirchng;
			}

			break;
		case 'l': //ciara
			switch(name.GetAt(5))
			{
			case 'l':
				properties=L_LEFT;
				break;
			case 'r':
				properties=L_RIGHT;
				break;
			case 't':
				properties=L_TOP;
				break;
			case 'b':
				properties=L_BOTTON;
				break;
			default:
				properties=L_EMPTY;
			}
			if(properties)
			{
				poleCiary[poc_ciara].properties=properties;
				poleCiary[poc_ciara].direction=dir;
				poleCiary[poc_ciara].distance=dist;
				poc_ciara++;
			}

			break;
		case 'g': //branka
			name.SetPosition(5);
			name.ParseString(T);
				poleFlags[poc_flag].direction=dir;
				poleFlags[poc_flag].distance=dist;
				poleFlags[poc_flag].prop1=F_GOALFLAG;
				poleFlags[poc_flag].prop2=FlagParams(T);;
				poleFlags[poc_flag].prop3=0;
				poc_flag++;
			break;
		case 'f': //flag
			name.SetPosition(5);
			DWORD p1=0;
			DWORD p2=0;
			DWORD p3=0;

			if(name.ParseString(T)) //1
			{
				p1=FlagParams(T);
				if(name.ParseString(T))
				{
					p2=FlagParams(T);
					if(name.ParseString(T))
					{
						p3=FlagParams(T);
					}
				}

				poleFlags[poc_flag].direction=dir;
				poleFlags[poc_flag].distance=dist;
				poleFlags[poc_flag].prop1=p1;
				poleFlags[poc_flag].prop2=p2;
				poleFlags[poc_flag].prop3=p3;
				poc_flag++;
			}
			break;
		}
	}

	VizInfo vinfo;
	vinfo.nCiary=poc_ciara;
	vinfo.Ciary=poleCiary;

	vinfo.nFlagy=poc_flag;
	vinfo.Flagy=poleFlags;

	vinfo.nNasi=poc_nasich;
	vinfo.Nasi=nasiHraci;

	vinfo.nCudzi=poc_cudzich;
	vinfo.Cudzi=cudziHraci;

	vinfo.nNeutralny=poc_neutral;
	vinfo.Neutralny=neutralHraci;

	vinfo.lpLopta=&Lopta;
	Handler_See(&vinfo,seetime);
}

void CLowAgent::ParseSense_Body(LPCSTR message)
{
	TParseString msg(message);
	TParseString pom;
	CString tmp;

	int cas;
	static SenseBodyInfo sinf;
	ZeroMemory(&sinf,sizeof(sinf));

	msg.SetPosition(12);
	cas=msg.ParseInteger();
	if(msg.HasErrorOccured())
	{
		MsgB("!.!");
		return;
	}

	while(msg.ParseBracket(pom))
	{
		pom.SetPosition(0);
		switch(pom.GetAt(0))
		{
		case 'v': //view_mode
			pom.ParseString(tmp);
			pom.ParseString(tmp);

			if(tmp.Compare("low")==0)
			{
				sinf.vquality=low;
			}
			else
			{
				sinf.vquality=high;
			}

			pom.ParseString(tmp);
			if(tmp.Compare("normal")==0)
			{
				sinf.vwidth=normal;
			}
			else
			{
				if(tmp.Compare("wide")==0)
					sinf.vwidth=wide;
				else sinf.vwidth=narrow;
			}
			break;
		case 's': //stamina, speep, say
			pom.ParseString(tmp);
			switch(tmp.GetAt(1))
			{
			case 't': //stamina
				sinf.stamina=pom.ParseDouble();
				sinf.Effort=pom.ParseDouble();
				break;
			case 'p': //speed
				sinf.speed=pom.ParseDouble();
				break;
			case 'a': //say
				sinf.say_count=pom.ParseInteger();
				break;
			}
			break;
		case 'h': //head_angle
			pom.ParseString(tmp);
			sinf.head_angle=pom.ParseDouble();
			break;
		case 'k': //kick
			pom.ParseString(tmp);
			sinf.kick_count=pom.ParseInteger();
			break;
		case 't': //turn, turn_neck
			pom.ParseString(tmp);
			switch(pom.GetAt(4))
			{
			case ' ': //turn
				sinf.turn_count=pom.ParseInteger();
				break;
			case '_': //turn_neck
				sinf.turnneck_count=pom.ParseInteger();
				break;
			}
			break;
		case 'd': //dash
			pom.ParseString(tmp);
			sinf.dash_count=pom.ParseInteger();
			break;
		}
	}

	Handler_SenseBody(&sinf,cas);
}

void CLowAgent::ParseInit(LPCSTR message)
{
	TParseString msg(message);
	CString tmp;
	int number;
	int mode=PM_None;
	BOOL Left2Right;
	char* keciRozhodcu[]=PLAYMODE_STRINGS;

	// get the side
	if ((msg.GetLength() > 6) && (msg.GetAt(6) == 'r'))
        Left2Right=false;
	else
        Left2Right=true;

	// get the number of the player
	msg.SetPosition(8);
	number = msg.ParseInteger();
	if (msg.HasErrorOccured())
	{
		MsgB("!!");
        return;
	}

	// get the playmode
	msg.ParseString(tmp);
	while ((mode < NO_OF_PLAYMODES) && (tmp.Compare(keciRozhodcu[mode])))
        mode++;

	if (mode < NO_OF_PLAYMODES)
	{
        Handler_ChangeMode((PlayMode)mode,0);
	}

	m_TryingToConnect=false;
	m_IsConnected=TRUE;
	Handler_Login(number,Left2Right);
}

void CLowAgent::ParseReconnect(LPCSTR message)
{
	ParseInit(message);
}

//zmena hracieho rezimu
void CLowAgent::Handler_ChangeMode(PlayMode mod,int cas)
{
	char* k[]=PLAYMODE_STRINGS;
	CString dmsg;
	dmsg.Format("Hraci rezim: %s [cas=%d]\n",k[mod],cas);
	cPrint(dmsg,1);
	
}

//po nakonektoani sa na server
void CLowAgent::Handler_Login(int Cislo, BOOL lava_strana)
{
	CString m;
	m.Format("Cislo hraca=%d, ",Cislo);
	if(lava_strana)
	{
		m+="lava strana\n";
	}
	else
	{
		m+="prava strana\n";
	}
	cPrint(m,15);
}

//zacutie spravy
void CLowAgent::Handler_Hear(int direction, CString &sprava,int cas)
{
	CString m;
	m.Format("Sprava: dir=%d cas=%d %s\n",direction,cas,sprava);
	cPrint(m);
}

//vyslovi pokec
void CLowAgent::Say(CString &pokec)
{
	if(!m_IsConnected) return;
	
	CString msg;
	msg.Format("(say %s)",(LPCTSTR)pokec);
	SendMsg(msg);
}

DWORD CLowAgent::FlagParams(CString &str)
{
	if(str.Compare("l")==0) return F_LEFT;
	if(str.Compare("r")==0) return F_RIGHT;
	if(str.Compare("c")==0) return F_CENTER;
	if(str.Compare("t")==0) return F_TOP;
	if(str.Compare("b")==0) return F_BOTTON;

	if(str.Compare("g")==0) return F_GOAL;
	if(str.Compare("p")==0) return F_PENALTY;

	if(str.Compare("0")==0) return F_0;
	if(str.Compare("10")==0) return F_10;
	if(str.Compare("20")==0) return F_20;
	if(str.Compare("30")==0) return F_30;
	if(str.Compare("40")==0) return F_40;
	if(str.Compare("50")==0) return F_50;

	return 0;
}

void CLowAgent::Handler_See(VizInfo *vinfo,int cas)
{

}

void CLowAgent::Handler_SenseBody(SenseBodyInfo *sinf, int cas)
{

}

/*
	Vyziada zaslanie informacii o tomto agentovi (version 4.00)
	Netreba pri verzii 5.00
*/
void CLowAgent::SenseBody()
{
	if(!m_IsConnected) return;
	CString tmp("(sense_body)");
	SendMsg(tmp);
}

void CLowAgent::Catch(int direction)
{
	if(!m_IsConnected) return;
	CString msg;
	msg.Format("(catch %d)",direction);
	SendMsg(msg);
}

void CLowAgent::Change_View(ViewWidth vwidth)
{
	if(!m_IsConnected) return;
	CString msg;
	switch(vwidth)
	{
	case narrow:
		msg="(change_view narrow high)";
		break;
	case normal:
		msg="(change_view normal high)";
		break;
	case wide:
		msg="(change_view wide high)";
		break;
	}
	SendMsg(msg);
}

void CLowAgent::Dash(int power)
{
	if(!m_IsConnected) return;
	CString msg;
	msg.Format("(dash %d)",power);
	SendMsg(msg);
	Handler_Dash(power);
}

void CLowAgent::Kick(int power, int direction)
{
	if(!m_IsConnected) return;
	CString msg;
	msg.Format("(kick %d %d)",power,direction);
	SendMsg(msg);
}

void CLowAgent::Move(int x, int y)
{
	if(!m_IsConnected) return;
	CString msg;
	msg.Format("(move %d %d)",x,y);
	SendMsg(msg);
}

void CLowAgent::Turn(int moment)
{
	if(!m_IsConnected) return;
	CString msg;
	msg.Format("(turn %d)",moment);
	SendMsg(msg);
	Handler_Turn(moment);
}

void CLowAgent::Turn_Neck(int angle)
{
	if(!m_IsConnected) return;
	CString msg;
	msg.Format("(turn_neck %d)",angle);
	SendMsg(msg);
	Handler_TurnNeck(angle);
}

void CLowAgent::Handler_Turn(int moment)
{

}

void CLowAgent::Handler_TurnNeck(int angle)
{

}

void CLowAgent::Handler_Dash(int power)
{

}

DWORD CLowAgent::GetTicks()
{
	if(m_DebugMode)
	{
		return m_DebugTicks;
	}
	else
	{
		return GetTickCount();
	}
}

void CLowAgent::ParseReceive(LPCSTR buf)
{
	switch(buf[1])
	{
	case 'h': //hear
		ParseHear(buf);
		break;
	case 's': //see or sense_body
		if(buf[3]=='e')
		{
			ParseSee(buf);
		}
		else
		{
			ParseSense_Body(buf);
		}
		break;
	case 'i': //init
		ParseInit(buf);
		break;
	case 'r': //reconnect
		ParseReconnect(buf);
		break;
	case 'e': //error
		CString m(buf);
		m+="\n";
		cPrint(m,4);
		break;
	}
}

void CLowAgent::ReceiveLogEvent()
{
	if(!m_DebugMode)
	{
		MsgB("Agent nie je v debug mode!");
		return;
	}

	if(m_DebugLogIndex > m_DebugLog.LogDB.GetUpperBound())
	{
		MsgB("Cela databaza bola prehrata");
		return;
	}

	m_DebugTicks= m_DebugLog.LogDB[m_DebugLogIndex].ticks;
	ParseReceive(m_DebugLog.LogDB[m_DebugLogIndex].message);
	m_DebugLogIndex++;
}
