// MedAgent.cpp: implementation of the CMedAgent class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "sharks.h"
#include "SharkTypes.h"
#include "MedAgent.h"
#include "TParseString.h"
#include "NetDlg.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMedAgent::CMedAgent()
{
	sync_SimTime=0;
	sync_LastVI_SimTime=0;
	sync_LastComm_TickCount=GetTicks();
	IsWaitingForVI=TRUE;
	WillWaitForVI=TRUE;

	sync_TimerNumber=0;

	last_angle=0;
	last_moment=0;
	last_power=0;

	m_output=NULL;

	Last_catchCycle=0;
	LastPassBallCycle=0;
}

CMedAgent::~CMedAgent()
{

}

void CMedAgent::Handler_See(VizInfo *vinfo, int cas)
{

	unsigned Posun;
	
	if(cas > sync_SimTime)
	{
		ApplyLastCommands();
		//update o Posun krokov
		Posun=cas-sync_SimTime;
		stepWorld(Posun);
		updateWorld(vinfo);

		IsWaitingForVI=FALSE;
		WillWaitForVI=FALSE;

		sync_SimTime=cas;
		sync_LastComm_TickCount=GetTicks();
		Handler_NewCycle();
		//cPrint(CString("n"));
	}
	else
	{
		if(cas > sync_LastVI_SimTime) //nova vinfo, ale uz bol novy cyklus SB
		{
			//update bez step
			Posun=0;
			updateWorld(vinfo);

			if(IsWaitingForVI)
			{
				WillWaitForVI=FALSE;
				IsWaitingForVI=FALSE;
				sync_LastComm_TickCount=GetTicks();
				Handler_NewCycle();
			}
		}
		else
		{
			DWORD rozdiel;
			rozdiel=GetTicks() - sync_LastComm_TickCount;
			Posun=rozdiel / simulator_step;
			if(Posun>10) Posun=10; //kvoli load z logu
			if(rozdiel > simulator_step+55) //ak je zastaveny cas
			{
				ApplyLastCommands();
				//update o predpokladany pocet krokov
				stepWorld(Posun);
				updateWorld(vinfo);

				IsWaitingForVI=FALSE;
				WillWaitForVI=FALSE;
				sync_LastComm_TickCount=GetTicks();
				Handler_NewCycle();
				//cPrint(CString("r"));
			}
		}

	}

	sync_LastVI_SimTime=cas;

}

void CMedAgent::KickToGoal()
{
	int dir;
	double dist;
	double targetx;

	if(ourLeftSide)
	{
		targetx=FWIDTH;
	}
	else
	{
		targetx=-FWIDTH;
	}

	Calc_DirDist(targetx-Me.MyPosition.x,-Me.MyPosition.y,Me.body_angle,&dir,&dist);
	Kick(100,dir);
}

void CMedAgent::Handler_SenseBody(SenseBodyInfo* sinf,int cas)
{
	unsigned Posun;

	if(cas > sync_SimTime)
	{
		ApplyLastCommands();
		//iba step
		Posun=cas-sync_SimTime;
		stepWorld(Posun);
		updateMyBody(sinf);
	
		sync_SimTime=cas;
		sync_LastComm_TickCount=GetTicks();

		if(!IsWaitingForVI)
		{
			Handler_NewCycle();
			//cPrint(CString("S"));
		}
	}
	else
	{
		DWORD rozdiel;
		rozdiel=GetTicks() - sync_LastComm_TickCount;
		
		if(rozdiel > simulator_step +55) //cas je zastaveny
		{
			ApplyLastCommands();
		}

		//CString s;
		//s.Format("SPEED %.2f %.2f\n",Me.speed,sinf->speed);
		//cPrint(s,15);
		updateMyBody(sinf);
	}
}

void CMedAgent::Calc_DirDist(double dX, double dY,double sDir, int *dir, double *dist)
{
	int uD;
	uD=int(atan2(dY,dX)*180/PICKO - sDir);

	while(uD>180)  uD-=360;
	while(uD<-180) uD+=360;

	*dir=uD;
	*dist=sqrt(dX*dX+dY*dY);
}

BOOL CMedAgent::RunTo(double destX, double destY)
{
	int dir;
	double dist;

	Calc_DirDist(destX-Me.MyPosition.x,destY-Me.MyPosition.y,Me.body_angle,&dir,&dist);

	if(dist<0.5) return FALSE;

	if(abs(dir)>5) //otacame
	{
		Turn(Calc_Moment(dir));
	}
	else
	{//bezime
		if(dist>1.5)
		{
			Dash(100);
		}
		else if(dist<1)
		{
			Dash(50);
		}
		else //medzi 1.0 a 1.5
		{
			Dash(int((dist-1)*100)+50);
		}
	}
	return TRUE;
}

void CMedAgent::SayMyPos()
{
	CString m;
	m.Format("SHARKs_pos_%.1f_%.1f",Me.MyPosition.x,Me.MyPosition.y);
	Say(m);
}

void CMedAgent::Handler_Hear(int direction,CString& sprava,int cas)
{
	int dlzka=sprava.GetLength();
	TParseString msg(sprava);

	for(int i=0;i<dlzka;i++)
	{
		if(msg.GetAt(i) == '_')
			msg.SetAt(i,' ');
	}

	CString tmp;
	if(!msg.ParseString(tmp)) return;
	if(tmp.Compare("SHARKs")!=0) return;

	if(!msg.ParseString(tmp)) return;
	if(tmp.Compare("pos")!=0) return;

	double hX,hY;
	hX=msg.ParseDouble();
	if(msg.HasErrorOccured()) return;
	hY=msg.ParseDouble();
	if(msg.HasErrorOccured()) return;

	CString s;
	s.Format("Detekovana pozicia %f %f\n",hX,hY);
	cPrint(s,13);
}

void CMedAgent::CreateTimer(CWnd *okno)
{
	if(sync_TimerNumber) return;
	sync_LastComm_TickCount=GetTicks();
	sync_TimerNumber=okno->SetTimer(1,CConfigFile::simulator_step,NULL);
}

BOOL CMedAgent::OnExternTimer()
{
	if(!m_IsConnected && !m_TryingToConnect) return TRUE;

	DWORD rozdiel;
	rozdiel=GetTicks() - sync_LastComm_TickCount;

	float cyklov=float(rozdiel)/float(simulator_step);
	if(cyklov > 20.0f) return FALSE;

	if(m_TryingToConnect) return TRUE;

	if(cyklov > 1.0f)
	{
		SenseBody();
	}
	return TRUE;
}

void CMedAgent::RunToBall()
{
	//stary algoritmus
	//RunTo(Lopta.MyPosition.x,Lopta.MyPosition.y);

	int r_dir;	//predpokladany uhol
	double r_dist; //predp. vzial
	int psteps; //predp. pocet krokov

	int i;
	CPlayer tmpLopta(Lopta);

	#define PREDICT_MAX	15
	for(i=1;i<=PREDICT_MAX;i++)
	{
		tmpLopta=stepBall(tmpLopta,1);
		Calc_DirDist(tmpLopta.MyPosition.x-Me.MyPosition.x,tmpLopta.MyPosition.y-Me.MyPosition.y,Me.body_angle,&r_dir,&r_dist);
		if(abs(r_dir) >5)
		{
			//psteps=(r_dist + __max(0,player_speed_max - maxpower*dash_power_rate))/(player_speed_max) + 1; //musi sa aj otocit a rozbiehat
			psteps=PredictSteps(r_dist,Me.speed*player_decay)+1;
		}
		else
		{
			//psteps=(r_dist)/(player_speed_max);
			psteps=PredictSteps(r_dist,Me.speed);
		}
		if(psteps<i || i==PREDICT_MAX) //ak tam moze dojst za i krokov
		{
			RunTo(tmpLopta.MyPosition.x,tmpLopta.MyPosition.y);
			return;
		}
	}

}

void CMedAgent::Handler_NewCycle()
{

}

BOOL CMedAgent::IsBallKickAble()
{
	if(Lopta.probability==0) return FALSE;

	double dist,x,y;
	x=Lopta.MyPosition.x-Me.MyPosition.x;
	y=Lopta.MyPosition.y-Me.MyPosition.y;
	dist=sqrt((x*x)+(y*y));

	if(dist <= ball_size + player_size + kickable_margin)
		return TRUE;
	else
		return FALSE;
}

void CMedAgent::Handler_Turn(int moment)
{
	last_moment+=moment;
}

void CMedAgent::Handler_Dash(int power)
{
	last_power=power;
}

void CMedAgent::Handler_TurnNeck(int angle)
{
	last_angle+=angle;
}

/*
	Zobraz dialog, nakonektuje sa na server
	VRACIA:
		true	- uspech
		false	- neuspech
*/
BOOL CMedAgent::ConnectAgent()
{
	int retval;
	BOOL ret;

	CNetDlg netdlg;
	netdlg.m_TeamName=defaultTeamName();
	netdlg.m_SocServer=defaultServerName();		//nazov pripojenia
	netdlg.m_SocPort=defaultServerSocket();

	ret = FALSE;

	while (!ret) {
		retval=netdlg.DoModal();
		if(retval==IDCANCEL) return false;
		ret = ConnectAgentEx(netdlg.m_TeamName, netdlg.m_SocServer, netdlg.m_SocPort, netdlg.m_GoalPerson);
	}

	return ret;
}

void CMedAgent::ApplyLastCommands()
{
	Me.body_angle+=(last_moment / (1+inertia_moment*fabs(Me.speed)));
	last_moment=0;
	if(Me.body_angle < -180) Me.body_angle+=360;
	if(Me.body_angle > 180 ) Me.body_angle-=360;

	Me.head_angle+=last_angle;
	last_angle=0;
	if(Me.head_angle < minneckang) Me.head_angle=minneckang;
	if(Me.head_angle > maxneckang) Me.head_angle=maxneckang;

	Me.speed= Me.speed + (last_power * dash_power_rate);
	if(Me.speed > player_speed_max)
	{
		Me.speed = player_speed_max;
	}

	last_power=0;

	IsWaitingForVI=WillWaitForVI;

}

void CMedAgent::SateliteScan()
{
	CString str;
	str.Format("Scanning_%d\n",sync_SimTime);
	cPrint(str,11);
	switch(Me.vwidth)
	{
	case narrow:
		Turn(40);
		break;
	case normal:
		Turn(80);
		break;
	case wide:
		Turn(160);
		break;
	}
	WillWaitForVI=TRUE;
}

BOOL CMedAgent::IsBallNearGoal(BOOL side)
{
	double x,dx;
	x= (side) ? -FWIDTH : FWIDTH;

	dx=Lopta.MyPosition.x - x;
	double d=sqrt(dx*dx + Lopta.MyPosition.y*Lopta.MyPosition.y);

	if(d<22.0)
		return TRUE;
	else
		return FALSE;
}

void CMedAgent::LookAtBall()
{
	double dist;
	int angleTo;

	Calc_DirDist(Lopta.MyPosition.x-Me.MyPosition.x,Lopta.MyPosition.y-Me.MyPosition.y,Me.body_angle,&angleTo,&dist);
	if(abs(angleTo)>0) Turn(angleTo);
}

BOOL CMedAgent::IsBallCatchAble()
{
	if(Lopta.probability == 0) return FALSE;
	if(sync_SimTime <= (Last_catchCycle + catch_ban_cycle)) return FALSE;

	if(Me.MyPosition.y < -PHEIGHT || Me.MyPosition.y > PHEIGHT) return FALSE;
	if(fabs(Me.MyPosition.x) < PWIDTH) return FALSE;

	if(Lopta.MyPosition.y < -PHEIGHT || Lopta.MyPosition.y > PHEIGHT) return FALSE;
	if(fabs(Lopta.MyPosition.x) < PWIDTH) return FALSE;

	double dx=fabs(Lopta.MyPosition.x - Me.MyPosition.x);
	double dy=fabs(Lopta.MyPosition.y - Me.MyPosition.y);

	if((dx < catchable_area_l/2.0f) && (dy < catchable_area_w/2.0f)) return TRUE;
	return FALSE;
}

void CMedAgent::Kresli()
{
	if(!m_output) return;
	CClientDC dc(m_output); // device context for painting
	CRect rect;
	m_output->GetClientRect(&rect);

	drawMemory(dc,rect);

	UINT flags=dc.SetTextAlign(TA_RIGHT|TA_TOP);

	CString str;
	str.Format("%d",sync_SimTime);
	dc.SetTextColor(RGB(32,16,64));
	dc.TextOut(rect.right,rect.top,str);

	dc.SetTextAlign(flags);
}

int CMedAgent::GetSimTime() const
{
	return sync_SimTime;
}

void CMedAgent::PassBallTo(const CPlayer &spoluHrac)
{
	//(Tx,Ty) - miesto kam kopne (sucet nekonecneho radu? MA1)
	double Tx=spoluHrac.MyPosition.x + spoluHrac.speedX/(1-player_decay);
	double Ty=spoluHrac.MyPosition.y + spoluHrac.speedY/(1-player_decay);

	int dir;
	double dist;
	Calc_DirDist(Tx - Lopta.MyPosition.x,Ty - Lopta.MyPosition.y,Me.body_angle,&dir,&dist);

	if(dist>20)
	{
		Kick(100,dir);
	}
	else if(dist<5)
	{
		Kick(50,dir);
	}
	else
	{
		int pow=int( (dist-5) * 50/15) + 50;
		Kick(pow,dir);
	}
}

//index do pola Nasi (najblizsi spoluhrac pri lopte)
int CMedAgent::CloserFriend2Ball(double *dist)
{
	if(Nasi.GetSize() == 0) return -1;

	int i;
	int closer=0;
	double max_dist=170; //nejake velke cislo
	double tmp_dist;

	for(i=0;i<Nasi.GetSize();i++)
	{
		double dx = Lopta.MyPosition.x - Nasi[i].MyPosition.x;
		double dy = Lopta.MyPosition.y - Nasi[i].MyPosition.y;
		tmp_dist=sqrt(dx*dx+dy*dy);
		
		if(tmp_dist < max_dist)
		{
			max_dist=tmp_dist;
			closer=i;
		}
	}

	*dist=max_dist;
	return closer;
}

int CMedAgent::CloserEnemy2Ball(double *dist)
{
	if(Cudzi.GetSize() == 0) return -1;

	int i;
	int closer=0;
	double max_dist=170; //nejake velke cislo
	double tmp_dist;

	for(i=0;i<Cudzi.GetSize();i++)
	{
		double dx = Lopta.MyPosition.x - Cudzi[i].MyPosition.x;
		double dy = Lopta.MyPosition.y - Cudzi[i].MyPosition.y;
		tmp_dist=sqrt(dx*dx+dy*dy);
		
		if(tmp_dist < max_dist)
		{
			max_dist=tmp_dist;
			closer=i;
		}
	}

	*dist=max_dist;
	return closer;
}


BOOL CMedAgent::Responsibility4Ball()
{
	int cfriend;
	double friend_dist;
	cfriend=CloserFriend2Ball(&friend_dist);
	if(cfriend==-1) return TRUE; //nevidim spoluhraca

	double dx = Lopta.MyPosition.x - Me.MyPosition.x;
	double dy = Lopta.MyPosition.y - Me.MyPosition.y;
	double MyDist=sqrt(dx*dx + dy*dy);

	if(MyDist < friend_dist) return TRUE;//som najblizsie

	int cenemy;
	double enemy_dist;
	cenemy=CloserEnemy2Ball(&enemy_dist);
	if(cenemy==-1) return FALSE;

	if(enemy_dist < friend_dist) //najblizsie je protihrac
	{
		dx= Nasi[cfriend].MyPosition.x - Me.MyPosition.x;
		dy= Nasi[cfriend].MyPosition.y - Me.MyPosition.y;

		//vzdialenos medzi tmto hrom, a hrom nho tmu, ktor je najbliie k lopte, je via ako vzdialenos spoluhra od lopty
		//najbliie k lopte je sperov hr
		if(sqrt(dx*dx+dy*dy) > friend_dist) return TRUE;

	}
	

	return FALSE;
}



int CMedAgent::FindPassAblePlayer()
{
	if(Nasi.GetSize()==0) return -1;

	double dx=(ourLeftSide ? FWIDTH : -FWIDTH) - Me.MyPosition.x;
	double dy=0-Me.MyPosition.y;
	double brana_vzdial=sqrt(dx*dx+dy*dy);

	int best=-1;

	for(int i=0;i<Nasi.GetSize();i++)
	{
		dx=Nasi[i].MyPosition.x-Me.MyPosition.x;
		dy=Nasi[i].MyPosition.y-Me.MyPosition.y;
		double vzdial_rel=sqrt(dx*dx+dy*dy);
		
		if(vzdial_rel <30 && vzdial_rel >3 && IsFree(i)) //aby nenahraval prilis daleko alebo blizko a obsadenemu
		{
			dx=(ourLeftSide ? FWIDTH : -FWIDTH) - Nasi[i].MyPosition.x;
			dy=0 - Nasi[i].MyPosition.y;
			double hrac_vzdial=sqrt(dx*dx+dy*dy);
			if(hrac_vzdial < brana_vzdial)
			{
				best=i;
				brana_vzdial=hrac_vzdial;
			}
		}
	}

	return best;
}

BOOL CMedAgent::IsFree(int spoluhrac)
{
	for(int i=0;i<Cudzi.GetSize();i++)
	{
		double dx=Cudzi[i].MyPosition.x-Nasi[spoluhrac].MyPosition.x;
		double dy=Cudzi[i].MyPosition.y-Nasi[spoluhrac].MyPosition.y;
		double dist=sqrt(dx*dx+dy*dy);
		if(dist<5) return FALSE;

	}
	return TRUE;
}

void CMedAgent::DoCatchBall()
{
	int dir;
	double dist;
	Calc_DirDist(Lopta.MyPosition.x - Me.MyPosition.x,Lopta.MyPosition.y - Me.MyPosition.y,Me.body_angle,&dir,&dist);
	Catch(dir);
	cPrint(CString("chytam loptu...\n"),2);
	Last_catchCycle=sync_SimTime;
}

BOOL CMedAgent::INIConnectAgent(CWnd *wnd)
{

	loadPositions();

	BOOL ret;
	ret = ConnectAgentEx(defaultTeamName(), defaultServerName(), defaultServerSocket(), defaultGoalie());

	if (ret) CreateTimer(wnd);
	return ret;
}



int CMedAgent::Calc_Moment(int angle)
{
	int moment = int(angle * (1 + inertia_moment*fabs(Me.speed)));
	if(moment < minmoment) moment=int(minmoment);
	if(moment > maxmoment) moment=int(maxmoment);

	return moment;
}

int CMedAgent::PredictSteps(double dist, double aktSpeed)
{
	double distance=dist;
	double speed_a=aktSpeed;

	int Steps=0;
	while(distance>0)
	{
		speed_a=__min(player_speed_max,speed_a + 100 * dash_power_rate);
		distance -= speed_a;
		speed_a *= player_decay;
		Steps++;
	}

	return Steps;
}

void CMedAgent::PassBallTo2(const CPlayer &spoluHrac)
{
	if(sync_SimTime == (LastPassBallCycle+1))
	{//pouzije sa stare kopnutie
		PassBallTo(spoluHrac);
		return;
	}
	
	LastPassBallCycle=sync_SimTime; //zapamatam kedy som kopol pomocou PassBallTo2

	//(Tx,Ty) - miesto kam kopne (sucet nekonecneho radu? MA1)
	double Tx=spoluHrac.MyPosition.x + spoluHrac.speedX/(1-player_decay);
	double Ty=spoluHrac.MyPosition.y + spoluHrac.speedY/(1-player_decay);

	double dx = Tx-Me.MyPosition.x;
	double dy = Ty-Me.MyPosition.y;
	double dist = sqrt(dx*dx+dy*dy);

	double rough_steps= dist/ball_speed_max; //dolny odhad poctu krokov ku spoluhracovi
	double pw = pow(ball_decay,rough_steps);
	ASSERT(pw>=0 && pw <= 1);

	Tx = Tx - Lopta.speedX * (1 - pw)/(1-ball_decay);
	Ty = Ty - Lopta.speedY * (1 - pw)/(1-ball_decay);

	int dir;
	Calc_DirDist(Tx - Lopta.MyPosition.x,Ty - Lopta.MyPosition.y,Me.body_angle,&dir,&dist);

	if(dist>20)
	{
		Kick(100,dir);
	}
	else if(dist<5)
	{
		Kick(50,dir);
	}
	else
	{
		int pow=int( (dist-5) * 50/15) + 50;
		Kick(pow,dir);
	}
}
